$Title Multi reservoir MINLP model.

Sets
    i /t1*t21/
    res /r1*r2/;

*Defining parameters for the model
Scalars

    R /0/
    Tax /0/
    a_drillex /5E8/
    a_capex /50E6/
    b_capex /8.56E5/
    c_capex /4.42E4/
    d_capex /2.35E2/
    e_capex /2.15E9/
    a_opex /723.3E6/
    b_opex /6.5E6/
    c_opex /2.36E4/
    d_opex /1.855E-12/
    e_opex /3.04E-15/
    disc /0.12/
    Nwf_year /3/
    fp /0.973/
    days /365/
    Np_init /0.0/
    Nw_init /3/
    xr /8.5/
    vc /6.29/
    Fc /1.0/
    Po /60/;

Parameter
    Nw_max(res)
        /r1 6,
        r2  3/
    qpp_well(res)
        /r1 4592.475,
        r2  2852.217/
    Nw_injprod(res)
        /r1 0.5,
        r2 0.33/
    a_dimpot(res)
        /r1 -39.413,
        r2 7067.6/
    b_dimpot(res)
        /r1 -13.968,
        r2 -3761.5/
    c_dimpot(res)
        /r1 6.2734,
        r2 553.01/
    d_dimpot(res)
        /r1 12.684,
        r2 21.825/
    e_dimpot(res)
        /r1 -6.7185,
        r2 -12.684/
    a_gor(res)
        /r1 556885.71,
        r2 11358985.64/
    b_gor(res)
        /r1 -353194.93,
        r2 -3064031.89/
    c_gor(res)
        /r1 92643.10,
        r2 288797.17/
    d_gor(res)
        /r1 -5277.34,
        r2 -5494.71/
    e_gor(res)
        /r1 316.34,
        r2 150.00/
    a_wc(res)
        /r1 -111.57,
        r2 -178.94/
    b_wc(res)
        /r1 119.26,
        r2  95.27/
    c_wc(res)
        /r1 -42.60,
        r2  -16.31/
    d_wc(res)
        /r1 6.84,
        r2 0.96/
    e_wc(res)
        /r1 0.011,
        r2 0.03/
    N(res)
        /r1 56.25e6
        r2 39.25e6/
    Fw(res)
        /r1 1.0
        r2 1.0/;
        
Parameter val(i);
val(i) = ord(i)-1; 

*Defining variables
Variables
    NPV;

Positive variables
    qo(res,i)
    qoMax
    qwMax
    qgMax
    qg(res,i)
    qw(res,i)
    Np(res,i)
    qpp(res,i)
    PV_d(i)
    PV_pre
    PV_o(i)
    PV_r(i);

Integer variables
    Nw(res,i);
    
Nw.up(res,i) = Nw_max(res);
Nw.fx('r1','t1') = 3;
Np.fx(res,'t1') = Np_init;

*Declaring equations
Equations
    OilRate
    GasRate
    WaterRate
    OilPotential
    OilRate_Bound
    NumWells_Bound
    FieldNumWells_Bound
    OilCapacity
    GasCapacity
    WaterCapacity
    DrillExCapEx_1
    DrillExCapEx
    PreCapEx
    OpEx
    Revenue
    Objective;

*Objective Function
Objective.. NPV =e= -PV_pre/1E3 + sum(i,PV_r(i+1) - PV_o(i+1) - PV_d(i))/1E3;
*Constraints
OilRate(res,i+1).. (qo(res,i+1) + qo(res,i))*0.5*days =e= Np(res,i+1) - Np(res,i);
GasRate(res,i).. qg(res,i) =e= qo(res,i) * (a_gor(res) * (Np(res,i)/N(res))**4
                            + b_gor(res) * (Np(res,i)/N(res))**3 + c_gor(res)
                            * (Np(res,i)/N(res))**2 + d_gor(res) * (Np(res,i)/N(res))
                            + e_gor(res));
WaterRate(res,i).. qw(res,i) =e= qo(res,i) * (a_wc(res) * (Np(res,i)/N(res))**4
                            + b_wc(res) * (Np(res,i)/N(res))**3 + c_wc(res)
                            * (Np(res,i)/N(res))**2 + d_wc(res) * (Np(res,i)/N(res))
                            + e_wc(res))/(1-(a_wc(res) * (Np(res,i)/N(res))**4
                            + b_wc(res) * (Np(res,i)/N(res))**3 + c_wc(res)
                            * (Np(res,i)/N(res))**2 + d_wc(res) * (Np(res,i)/N(res))
                            + e_wc(res)));
OilPotential(res,i).. qpp(res,i) =e= Fw(res) * Nw(res,i)*qpp_well(res)
                            * fp**(Nw(res,i)-1) * (a_dimpot(res) * (Np(res,i)/N(res))**5
                            + b_dimpot(res) * (Np(res,i)/N(res))**4 + c_dimpot(res)
                            * (Np(res,i)/N(res))**3 + d_dimpot(res) * (Np(res,i)/N(res))**2
                            + e_dimpot(res) * (Np(res,i)/N(res)) + 1);
OilRate_Bound(res,i).. qo(res,i) =l= qpp(res,i);
NumWells_Bound(res,i+1).. Nw(res,i+1) =g= Nw(res,i);
FieldNumWells_Bound(i+1).. sum(res, Nw(res,i+1)-Nw(res,i)) =l= Nwf_year;
OilCapacity(i).. sum(res, qo(res,i)) =l= qoMax;
GasCapacity(i).. sum(res, qg(res,i)) =l= qgMax;
WaterCapacity(i).. sum(res, qw(res,i)) =l= qwMax;
DrillExCapEx_1.. PV_d('t1') =e= Fc * sum(res, Nw(res,'t1')*(1+Nw_injprod(res)))
                            * (a_drillex+a_capex)/(1E3*(1 + disc)**val('t1'));
DrillExCapEx(i+1).. PV_d(i+1) =e= Fc * sum(res, (Nw(res,i+1) - Nw(res,i))
                                * (1+Nw_injprod(res))) * (a_drillex+a_capex)
                                /(1E3*(1 + disc)**val(i+1));
PreCapEx.. PV_pre =e= Fc * (qoMax * b_capex + qwMax * c_capex + qgMax * d_capex
                        + e_capex)/1E3;
OpEx(i+1).. PV_o(i+1) =e= Fc * (a_opex + b_opex * sum(res, Nw(res,i)) + c_opex
                        * sum(res, qo(res,i)) + d_opex * sum(res, qw(res,i))
                        + e_opex * sum(res, qg(res,i)))/(1E3*(1 + disc)**val(i+1));
Revenue(i+1).. PV_r(i+1) =e= (Po * sum(res, Np(res,i+1) - Np(res,i)) * xr * vc)
                        /(1E3*(1 + disc)**val(i+1));

*Generating model
Model mod3 /all/;

*Choosing subsolvers for milp and nlp problems
option EMP = LINDO;
option minlp = lINDO;

File emp / '%emp.info%' /;
put emp '* problem %gams.i%'/;
$onput
randvar N('r1') discrete 0.3 31E6 0.4 49E6 0.3 68E6
randvar N('r2') discrete 0.3 33E6 0.4 40E6 0.3 49E6
randvar Fc discrete 0.3 0.81 0.4 1 0.3 1.18
randvar Po discrete 0.3 40 0.4 60 0.3 81
randvar Fw('r1') discrete 0.3 0.86 0.4 1 0.3 1.13 
stage 2 qo qg qw Np qpp PV_d PV_pre PV_o PV_r NPV N Fc Fw Po
stage 2 OilRate GasRate WaterRate OilPotential OilRate_Bound OilCapacity GasCapacity
        WaterCapacity DrillExCapEx_1 DrillExCapEx PreCapEx OpEx Revenue
$offput
putclose emp;

Set s  "scenarios" /s1*s243/;
Parameter
    s_N(s,res)
    s_Fc(s)
    s_Fw(s,res)
    s_Po(s)
    s_Nw(s,res,i)
    s_qo(s,res,i)
    s_NPV(s)
    s_qoMax(s)
    s_qgMax(s)
    s_qwMax(s)
    s_rep(s,*) "scenario probability"   / #s.prob 0/;;

Set dict /  s       .scenario   .''
            N       .randvar    .s_N
            Fc      .randvar    .s_Fc
            Po      .randvar    .s_Po
            Fw      .randvar    .s_Fw
            qo      .level      .s_qo
            Nw      .level      .s_Nw
            NPV     .level      .s_NPV
            qoMax   .level      .s_qoMax
            qgMax   .level      .s_qgMax
            qwMax   .level      .s_qwMax
            ''   .opt     .s_rep/;
 
          
Solve mod3 maximizing NPV using EMP scenario dict;

display  s_NPV, s_rep, s_N, s_Fc, s_Po, s_Fw

execute_unload "results.gdx" s_NPV s_rep s_N s_Fc s_Po s_Fw s_Nw s_qoMax s_qgMax s_qwMax