$Title Multi reservoir MINLP model.

Sets
    i /t1*t21/
    k /N1, N2, Fc, Fw1, Fw2/
    res /r1*r2/;

*Defining parameters for the model
Scalars

    R /0/
    Tax /0/
    a_drillex /5E5/
    a_capex /50E3/
    b_capex /8.56E2/
    c_capex /4.42E1/
    d_capex /2.35E2/
    e_capex /2.15E6/
    a_opex /723.3E3/
    b_opex /6.5E3/
    c_opex /2.36E1/
    d_opex /1.855E-15/
    e_opex /3.04E-15/
    disc /0.12/
    Nwf_year /3/
    fp /0.973/
    days /365/
    Np_init /0.0/
    Nw_init /3/
    xr /8.5/
    vc /6.29/
    Fc /1.0/
    qoMax /8915.93/
    qgMax /5494.73/
    qwMax /4752.47/;

Parameter
    qpp_well(res)
        /r1 4592.475,
        r2  2852.217/
    Nw_injprod(res)
        /r1 0.5,
        r2 0.33/
    a_dimpot(res)
        /r1 -39.413,
        r2 7067.6/
    b_dimpot(res)
        /r1 -13.968,
        r2 -3761.5/
    c_dimpot(res)
        /r1 6.2734,
        r2 553.01/
    d_dimpot(res)
        /r1 12.684,
        r2 21.825/
    e_dimpot(res)
        /r1 -6.7185,
        r2 -12.684/
    a_gor(res)
        /r1 556885.71,
        r2 11358985.64/
    b_gor(res)
        /r1 -353194.93,
        r2 -3064031.89/
    c_gor(res)
        /r1 92643.10,
        r2 288797.17/
    d_gor(res)
        /r1 -5277.34,
        r2 -5494.71/
    e_gor(res)
        /r1 316.34,
        r2 150.00/
    a_wc(res)
        /r1 -111.57,
        r2 -178.94/
    b_wc(res)
        /r1 119.26,
        r2  95.27/
    c_wc(res)
        /r1 -42.60,
        r2  -16.31/
    d_wc(res)
        /r1 6.84,
        r2 0.96/
    e_wc(res)
        /r1 0.011,
        r2 0.03/
    N(res)
        /r1 56.25e6
        r2 39.25e6/
    Fw(res)
        /r1 1.0
        r2 1.0/
    Po(i)
        /t1	60
        t2	60
        t3	60
        t4	60
        t5	60
        t6	60
        t7	60
        t8	60
        t9	60
        t10	60
        t11	60
        t12	60
        t13	60
        t14	60
        t15	60
        t16	60
        t17	60
        t18	60
        t19	60
        t20	60/;

Table
    Nw_min(res,i)
        t1  t2  t3  t4  t5  t6  t7  t8  t9  t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 t21
    r1  3   3   3   4   4   5   5   5   5   5   5   5   5   5   5   5   5   5   5   5   5      
    r2  0   0   0   1   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3;
    
Table
    Nw_max(res,i)
        t1  t2  t3  t4  t5  t6  t7  t8  t9  t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 t21
    r1  3   4   6   6   6   6   6   6   6   6   6   6   6   6   6   6   6   6   6   6   6      
    r2  0   0   2   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3;
    
    
Parameter val(i);
val(i) = ord(i)-1; 

*Defining variables
Variables
    DCF(i)
    NPV;

Positive variables
    qo(res,i)
    qo_f(i)
*qoMax
*qwMax
*qgMax
    qg(res,i)
    qw(res,i)
    Np(res,i)
    Np_f(i)
    qpp(res,i)
    PV_d(i)
    PV_c(i)
    PV_pre
    PV_o(i)
    PV_r(i);

Integer variables
    Nw(res,i)
    Nw_add(res,i)
    Nw_f(i);
    
*Declaring equations
Equations
    a1
    a2
    a3
    a4
    a5
    a6
    a7
    a8
    a9
    a10
    a11
    a12
    a13
    a14
    a15
    a16
    a17
    a18
    a19
    a20
    a21
    a22
    a23
    a24
    a25
    a26
    a300
    obj;

*OBJECTIVE FUNCTION

obj.. NPV =e= (1-R)*(1-Tax)*NPV;

a1(res).. Np(res,'t1') =e= Np_init;
a2(res,i).. Nw(res,i) =e= Nw_min(res,i) + Nw_add(res,i);
a3(res).. sum(i,Nw_add(res,i)) =l= sum(i,Nw_max(res,i)-Nw_min(res,i));
a300(res,i).. Nw(res,i) =l= Nw_max(res,i);
a4(i).. Nw_f(i) =e= sum(res, Nw(res,i));
a5(res,i+1).. (qo(res,i+1) + qo(res,i))*0.5*days =e= Np(res,i+1) - Np(res,i);
a6(res,i).. qg(res,i) =e= qo(res,i) * (a_gor(res) * (Np(res,i)/N(res))**4 + b_gor(res)
                            * (Np(res,i)/N(res))**3 + c_gor(res) * (Np(res,i)/N(res))**2
                            + d_gor(res) * (Np(res,i)/N(res)) + e_gor(res))/1E3;
a7(res,i).. qw(res,i) =e= qo(res,i) * (a_wc(res) * (Np(res,i)/N(res))**4 + b_wc(res)
                            * (Np(res,i)/N(res))**3 + c_wc(res) * (Np(res,i)/N(res))**2
                            + d_wc(res) * (Np(res,i)/N(res)) + e_wc(res))/(1-(a_wc(res)
                            * (Np(res,i)/N(res))**4 + b_wc(res) * (Np(res,i)/N(res))**3
                            + c_wc(res) * (Np(res,i)/N(res))**2 + d_wc(res)
                            * (Np(res,i)/N(res)) + e_wc(res)));
a8(i).. qo_f(i) =e= sum(res, qo(res,i));
a9(i).. Np_f(i) =e= sum(res, Np(res,i));
a10(res,i).. qpp(res,i) =e= Fw(res) * Nw(res,i)*qpp_well(res) * fp**(Nw(res,i)-1) *
                            (a_dimpot(res) * (Np(res,i)/N(res))**5 + b_dimpot(res)
                            * (Np(res,i)/N(res))**4 + c_dimpot(res) * (Np(res,i)/N(res))**3
                            + d_dimpot(res) * (Np(res,i)/N(res))**2
                            + e_dimpot(res) * (Np(res,i)/N(res)) + 1);
a11(res,i).. qo(res,i) =l= qpp(res,i);
a12(res,i+1).. Nw(res,i+1) =g= Nw(res,i);
a13(i+1).. Nw_f(i+1) - Nw_f(i) =l= Nwf_year;
a14(i).. qo_f(i) =l= qoMax;
a15(i).. sum(res, qg(res,i)) =l= qgMax;
a16(i).. sum(res, qw(res,i)) =l= qwMax;
a17.. PV_d('t1') =e= Fc * sum(res, Nw(res,'t1')*(1+Nw_injprod(res))) * a_drillex
                        /(1 + disc)**val('t1');
a18(i+1).. PV_d(i+1) =e= Fc * sum(res, (Nw(res,i+1) - Nw(res,i))*(1+Nw_injprod(res)))
                        * a_drillex/(1 + disc)**val(i+1);
a19.. PV_c('t1') =e= Fc * sum(res, Nw_max(res,'t1')*(1+Nw_injprod(res)))
                        * a_capex/(1+disc)**val('t1');
a20(i+1).. PV_c(i+1) =e= Fc * sum(res, (Nw_max(res,i+1) - Nw_max(res,i))
                        *(1+Nw_injprod(res))) * a_capex/(1 + disc)**val(i+1);
a21.. PV_pre =e= Fc * (qoMax * b_capex + qwMax * c_capex + qgMax * d_capex + e_capex);
a22(i+1).. PV_o(i+1) =e= Fc * (a_opex + b_opex * Nw_f(i) + c_opex * qo_f(i)
                        + d_opex * sum(res, qw(res,i)) + e_opex
                        * sum(res, qg(res,i)))/(1 + disc)**val(i+1);
a23(i+1).. PV_r(i+1) =e= (Po(i) * (Np_f(i+1) - Np_f(i)) * xr * vc)/(1 + disc)**val(i+1)/1E3;
a24.. DCF('t1') =e= -(PV_pre + PV_d('t1') + PV_c('t1'));
a25(i+1).. DCF(i+1) =e= (PV_r(i+1) - PV_o(i+1) - PV_c(i+1) - PV_d(i+1));
a26.. NPV =e= sum(i,DCF(i))/1E3;

*Generating model
Model mod3 /all/;

option minlp = lindo;

set s /s1*s400/

$call gdxxrw.exe LHS.xlsx par=LHS rng=Sheet1!A1:F401
Parameter LHS(s,k);
$gdxin LHS.gdx
$load LHS
$gdxin 

Parameter N_s(s,res), Fc_s(s), Fw_s(s,res);
N_s(s,'r1') = LHS(s,'N1')*1e6;
N_s(s,'r2') = LHS(s,'N2')*1e6;
Fc_s(s) = LHS(s, 'Fc');
Fw_s(s,'r1') = LHS(s, 'Fw1');
Fw_s(s,'r2') = LHS(s, 'Fw2');

$call gdxxrw.exe OilPriceTrajectory.xlsx par=oilprice rng=Sheet2!B2:W402
Parameter oilprice(s,i);
$gdxin OilPriceTrajectory.gdx
$load oilprice
$gdxin

Parameter qoresult(s,res,i);
Parameter qppresult(s,res,i);
Parameter qgresult(s,res,i);
Parameter qwresult(s,res,i);
Parameter Nwresult(s,res,i);
Parameter Npresult(s,res,i);
Parameter NPVresult(s);

Set mattrib / system.GUSSModelAttributes /;

Parameter
   srep(s, mattrib)                 'model attibutes like modelstat etc'
   o(*)/ SkipBaseCase 0
   RestartType 2/;

Set dict /  s.scenario.''
            o.  opt     .srep
            Po. param   .oilprice
            N.  param   .N_s
            Fc. param   .Fc_s
            Fw. param   .Fw_s
            qo. level   .qoresult
            qpp. level  .qppresult
            qg. level   .qgresult
            qw. level   .qwresult
            Nw. level   .Nwresult
            Np. level   .Npresult
            NPV. level   .NPVresult/;
 
          
Solve mod3 maximizing NPV using MINLP scenario dict;

execute_unload "results.gdx" qoresult qppresult qgresult qwresult Nwresult Npresult NPVresult

*=== Now write to variable levels to Excel file from GDX 
*=== Since we do not specify a sheet, data is placed in first sheet
execute 'gdxxrw.exe results.gdx o=results1.xlsx par=qoresult rng=qo!'
execute 'gdxxrw.exe results.gdx o=results1.xlsx par=qgresult rng=qg!'
execute 'gdxxrw.exe results.gdx o=results1.xlsx par=qwresult rng=qw!'
execute 'gdxxrw.exe results.gdx o=results1.xlsx par=Nwresult rng=nw!'
execute 'gdxxrw.exe results.gdx o=results1.xlsx par=NPVresult rng=npv!'